﻿using System;
using System.Collections.Generic;

namespace Phonebook
{
	/// <summary>
	/// Описва адресна книга
	/// </summary>
	public class Book
	{
		/// <summary>
		/// Вътрешен списък със записи. Думата "readonly" подсигурява,
		/// че той се създава само и единствено в конструкора и не се променя
		/// по време на жизнения цикъл на обекта
		/// </summary>
		private readonly List<Entry> _entries;

		/// <summary>
		/// Конструира телефонна книга.
		/// </summary>
		public Book()
		{
			// Създава вътрешния списък със записи в книгата
			this._entries = new List<Entry>();
		}

		/// <summary>
		/// Намира всички записи с посочен e-mail адрес.
		/// </summary>
		/// <param name="email">e-mail адрес, по който се търси</param>
		/// <returns>списък с намерени записи в книгата</returns>
		public List<Entry> FindByEmail(string email)
		{
			// списък с намерени адреси
			var found = new List<Entry>();

			// обхожда всички записи един по един
			foreach (Entry entry in this._entries)
			{
				// ако e-mail адресът съвпада с търсения,
				// се добавя в списъка с намерени адреси
				if (entry.Email == email)
					found.Add(entry);
			}

			// превръща списъка
			return found;
		}

		/// <summary>
		/// Търси записи по посочено име.
		/// </summary>
		/// <param name="name">име, по което се търси</param>
		/// <returns>списък с намерени записи в книгата</returns>
		/// <remarks>търсенето се осъществява по начин сходен като в <see cref="FindByEmail"/></remarks>
		public List<Entry> FindByName(string name)
		{
			var found = new List<Entry>();
			foreach (Entry entry in this._entries)
			{
				if (entry.Name == name)
					found.Add(entry);
			}

			return found;
		}

		/// <summary>
		/// Търси записи по посочен номер и евентуално вид на номера.
		/// </summary>
		/// <param name="number">номер, по който се търси</param>
		/// <param name="type">вид на номера, null - ако не се взима предвид</param>
		/// <returns>списък с намерени записи в книгата</returns>
		public List<Entry> FindByPhone(string number, PhoneType? type = null)
		{
			// създава списък с намерени записи
			var found = new List<Entry>();

			// обхожда записите в книгата един по един
			foreach (Entry entry in this._entries)
			{
				bool isMatch = false;

				// обхожда телефоните за текущия запис един по един
				foreach (Phone phone in entry.Phones)
				{
					// ако номерът съвпада...
					if (phone.Number == number)
					{
						// ако видът на номера не ни интересува,
						// отбелязваме, че имаме попадение
						if (type == null)
							isMatch = true;
						// ако видът на номера ни интересува и съвпада,
						// отбелязваме, че имаме попадение
						else if (phone.Type == type.Value)
							isMatch = true;
					}
				}

				// ако имаме попадение, добавяме настоящия запис
				// към списъка с намерените записи
				if (isMatch)
					found.Add(entry);
			}

			return found;
		}

		/// <summary>
		/// Добавя запис към книгата.
		/// </summary>
		/// <param name="entry">запис, който трябва да бъде добавен</param>
		public void AddEntry(Entry entry)
		{
			this._entries.Add(entry);
		}

		/// <summary>
		/// Изтрива запис от книгата.
		/// </summary>
		/// <param name="entry">запис, който трябва да бъде изтрит</param>
		public void DeleteEntry(Entry entry)
		{
			this._entries.Remove(entry);
		}

		/// <summary>
		/// Записва книгата.
		/// </summary>
		public void Save()
		{
			throw new NotImplementedException();
		}

		/// <summary>
		/// Разпечатва съдържането на книгата в конзолата.
		/// </summary>
		public void Dump()
		{
			foreach (Entry entry in this._entries)
			{
				Console.WriteLine("Entry:");
				Console.WriteLine("* E-mail: {0}", entry.Email);
				Console.WriteLine("* Name: {0}", entry.Name);
				Console.WriteLine("* Birthday: {0:D}", entry.Birthday);
				foreach (Phone phone in entry.Phones)
				{
					Console.WriteLine("* Phones:");
					Console.WriteLine("* * Type: {0}", phone.Type);
					Console.WriteLine("* * Number: {0}", phone.Number);
				}
			}
		}
	}
}
